package numericalmethod.cauchyproblem;

import numericalmethod.cauchyproblem.utils.Initialization;
import numericalmethod.cauchyproblem.utils.Plotting;
import numericalmethod.cauchyproblem.rungekutta.RK4;

/**
 * Класс для решения задачи Коши однородной системы дифференциальных уравнений I-ого порядка
 * методом Рунге-Кутта 4-ого порядка и вывод на экран результата сходимости значений произвольной координаты решений
 * в зависимости от момента времени для двух различных начальных условий
 */
public class Main {
    /**
     * Вспомогательный класс для хранения:
     * 1) массива значений j-ой координаты решений относительно момента времени;
     * 2) массива значений математического ожидания относительно момента времени.
     */
    private class Answer {
        double[] x_j;
        double[] mathE;

        private double[] getX_j() {
            return x_j;
        }

        private double[] getMathE() {
            return mathE;
        }

        private Answer(double[] x_j, double[] mathE) {
            this.x_j = x_j;
            this.mathE = mathE;
        }
    }

    private static final double t0 = 0d;   // При необходимости изменить начальный момент времени
    private static final double t1 = 18d;  // При необходимости изменить конечный момент времени
    private static final double h = 0.001; // При необходимости изменить шаг времени
    private static int stepCount = (int) ((t1 - t0) / h); // Количество шагов времени

    private static final int n = 200; // При необходимости изменить размерность квадратной матрицы

    // При необходимости изменить индекс ненулевой координаты первого начального вектора
    private static final int indexNonZeroCoordinateFirstInitialVector = 0;
    // При необходимости изменить индекс ненулевой координаты второго начального вектора
    private static final int indexNonZeroCoordinateSecondInitialVector = n;

    private static final int j = 0; // При необходимости изменить индекс координаты вектора решения

    // При необходимости изменить границу возмущения решения
    private static final double solutionPerturbationBounds = 0.0015;
    // При необходимости изменить границу возмущения матожидания
    private static final double mathExpectationPerturbationBounds = 0.15;

    private double[] t;
    private Initialization initialization;

    private numericalmethod.cauchyproblem.rungekutta.RK4 RK4;

    public static void main(String[] args) {
        new Main().run();
    }

    @SuppressWarnings("unchecked")
    private void run() {
        initialization = new Initialization(t0, h, stepCount,
                n, indexNonZeroCoordinateFirstInitialVector, indexNonZeroCoordinateSecondInitialVector);
        t = initialization.getT();
        double[] x1 = initialization.getX1(), x2 = initialization.getX2();

        RK4 = new RK4(n + 1, h);
        Answer ans1 = getAnswerByRK4(j, t, x1);
        Answer ans2 = getAnswerByRK4(j, t, x2);

        printDifference(ans1.x_j, ans2.x_j);
        plotResult(n + 1, t, ans1, ans2);
    }

    private Answer getAnswerByRK4(int j, double[] t, double[] x0) {
        double[] x1, x_j = new double[stepCount + 1];
        double[] mathE = new double[stepCount + 1];

        for (int i = 0; i <= stepCount; i++) {
            x_j[i] = x0[j];
            mathE[i] = getMathExpectation(x0);
            x1 = RK4.getNextVectorValue(initialization, t[i], x0);

            x0 = x1;
        }
        x_j[stepCount] = x0[j];
        mathE[stepCount] = getMathExpectation(x0);
        return new Answer(x_j, mathE);
    }

    private double getMathExpectation(double[] x) {
        double result = 0d;
        int i = 0;
        for (double x_i : x) {
            result += x_i * i;
            i++;
        }
        return result;
    }

    private void printDifference(double[] x1_j, double[] x2_j) {
        for (int i = 0; i <= stepCount; i++) {
            System.out.printf("t = %7.3f   ", t[i]);
            System.out.println("eps(|x1_" + j + " - x2_" + j + "|) = " + Math.abs(x1_j[i] - x2_j[i]));
        }
    }

    private void plotResult(int n, double[] t, Answer ans1, Answer ans2) {
        Plotting plotting = new Plotting();
        plotSolutionGraphics(n, t, ans1, ans2, plotting);
        plotMathExpectationGraphics(n, t, ans1, ans2, plotting);
        plotSolutionGraphicsWithBoard(n, t, solutionPerturbationBounds, ans1, plotting);
        plotMathExpectationGraphicsWithBoard(n, t, mathExpectationPerturbationBounds, ans1, plotting);
    }

    private void plotSolutionGraphics(int n, double[] t, Answer ans1, Answer ans2, Plotting plotting) {
        // Отображение графиков решений (по j-ой координате)
        plotting.showTwoGraphics(
                "Решение задачи Коши ОСДУ I-ого порядка (n = " + (n - 1) + ")",
                "t", "x(t)",
                "X(0) = " + indexNonZeroCoordinateFirstInitialVector, "X(0) = " + indexNonZeroCoordinateSecondInitialVector,
                t, ans1.getX_j(), ans2.getX_j(),
                t1, 1);
    }

    private void plotMathExpectationGraphics(int n, double[] t, Answer ans1, Answer ans2, Plotting plotting) {
        // Отображение графиков матожидания решений (по j-ой координате)
        plotting.showTwoGraphics(
                "Мат.ожидание от решения задачи Коши ОСДУ I-ого порядка (n = " + (n - 1) + ")",
                "t", "E(x(t))",
                "E[X(0) = " + indexNonZeroCoordinateFirstInitialVector + "]", "E[X(0) = " + indexNonZeroCoordinateSecondInitialVector + "]",
                t, ans1.getMathE(), ans2.getMathE(),
                t1, n);
    }

    private void plotSolutionGraphicsWithBoard(int n, double[] t, double eps, Answer ans, Plotting plotting) {
        plotting.showGraphicsWithBoard(
                "Решение задачи Коши ОСДУ I-ого порядка (n = " + (n - 1) + ") с границами устойчивости",
                "t", "x(t)",
                "X(0) = " + indexNonZeroCoordinateFirstInitialVector,
                t, ans.getX_j(), eps,
                t1);
    }

    private void plotMathExpectationGraphicsWithBoard(int n, double[] t, double eps, Answer ans, Plotting plotting) {
        plotting.showGraphicsWithBoard(
                "Мат.ожидание от решения задачи Коши ОСДУ I-ого порядка (n = " + (n - 1) + ") с границами устойчивости",
                "t", "E(x(t))",
                "E[X(0) = " + indexNonZeroCoordinateFirstInitialVector + "]",
                t, ans.getMathE(), eps,
                t1);
    }
}
