package numericalmethod.cauchyproblem.rungekutta;

import numericalmethod.cauchyproblem.utils.Initialization;

/**
 * Класс для получения значения функции в момент времени t через предыдущее значение функции (в момент времени t-h)
 * методом Рунге-Кутта 4ого порядка
 */
public class RK4 {
    private int n; // размерность вектора решения
    private double h; // длина шага времени
    private double half_h; // половина длины шага времени

    public RK4(int n, double h) {
        this.n = n;
        this.h = h;
        half_h = h / 2d;
    }

    /**
     * Получение значения функции x1 в момент времени t через предыдущее значение функии x0
     * @param t текущий момент времени
     * @param x0 значение функции в предыдущий момент времени (t-h)
     * @return значение функции в момент времени t
     */
    public double[] getNextVectorValue(Initialization initialization, double t, double[] x0) {
        double[] y1 = new double[n];
        double[] k1 = initialization.getF(t, x0);
        double[] k2 = initialization.getF(t + half_h, getX_with_halfH_and_K1(x0, k1));
        double[] k3 = initialization.getF(t + half_h, getX_with_halfH_and_K2(x0, k2));
        double[] k4 = initialization.getF(t + h, getX_with_halfH_and_K3(x0, k3));
        for (int i = 0; i < n; i++) {
            y1[i] = x0[i] + (k1[i] + 2d * k2[i] + 2d * k3[i] + k4[i]) * h / 6d;
        }
        return y1;
    }

    private double[] getX_with_halfH_and_K1(double[] x, double[] k1) {
        double[] xWithK1 = new double[n];
        for (int i = 0; i < n; i++) {
            xWithK1[i] = x[i] + half_h * k1[i];
        }
        return xWithK1;
    }

    private double[] getX_with_halfH_and_K2(double[] x, double[] k2) {
        double[] xWithK2 = new double[n];
        for (int i = 0; i < n; i++) {
            xWithK2[i] = x[i] + half_h * k2[i];
        }
        return xWithK2;
    }

    private double[] getX_with_halfH_and_K3(double[] x, double[] k3) {
        double[] xWithK3 = new double[n];
        for (int i = 0; i < n; i++) {
            xWithK3[i] = x[i] + h * k3[i];
        }
        return xWithK3;
    }
}
