package numericalmethod.cauchyproblem.utils;

import org.jfree.chart.*;
import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.block.BlockBorder;

import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.chart.ui.RectangleAnchor;
import org.jfree.chart.ui.RectangleEdge;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

public class Plotting {
    /**
     * Отрисовка двух графиков в новом окне
     * @param title наименование графика
     * @param xLabel наименование оси абсцисс
     * @param yLabel наименование оси ординат
     * @param graphName1 наименование первого графика
     * @param graphName2 наименование второго графика
     * @param t массив временных точек
     * @param x1 массив значений первого решения, в зависимости от временных точек из t
     * @param x2 массив значений второго решения, в зависимости от временных точек из t
     * @param tMaxBound максимальное значение времени
     * @param xMaxBound максимальное значение решения
     */
    public void showTwoGraphics(String title,
                                String xLabel, String yLabel,
                                String graphName1, String graphName2,
                                double[] t, double[] x1, double[] x2,
                                double tMaxBound, double xMaxBound) {

        JFreeChart chart = ChartFactory.createXYLineChart(
                title,
                xLabel,
                yLabel,
                initGraphicsCollection(graphName1, graphName2, t, x1, x2),
                PlotOrientation.VERTICAL,
                true,
                true,
                false);

        initWindowContentStyle(chart);
        initCoordinateGridStyle(tMaxBound, xMaxBound, chart);
        initWindowStile(chart);
    }

    public void showGraphicsWithBoard(String title,
                                String xLabel, String yLabel,
                                String graphName1,
                                double[] t, double[] x,
                                double eps,
                                double tMaxBound) {

        double[] majorX = new double[x.length], minorX = new double[x.length];
        for (int i = 0; i < x.length; i++) {
            majorX[i] = x[i] + eps;
            minorX[i] = (x[i] - eps >= 0 ? x[i] - eps : 0);
        }

        JFreeChart chart = ChartFactory.createXYLineChart(
                title,
                xLabel,
                yLabel,
                initGraphicsCollection(graphName1, t, x, majorX, minorX),
                PlotOrientation.VERTICAL,
                true,
                true,
                false);

        initWindowStile(chart);
        initCoordinateGridStyle(chart);
        initWindowContentStyle(chart);

        XYPlot xyPlot = chart.getXYPlot();
        // Задание верхних границ координатных осей
        ValueAxis domainAxis = xyPlot.getDomainAxis();
        domainAxis.setLowerBound(tMaxBound - 1d);
        domainAxis.setUpperBound(tMaxBound);

        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setSeriesShapesVisible(0, false);
        renderer.setSeriesPaint(0, Color.BLACK);
        renderer.setSeriesPaint(1, Color.GRAY);
        renderer.setSeriesShapesVisible(1, false);
        renderer.setSeriesVisibleInLegend(1,Boolean.FALSE);
        renderer.setSeriesPaint(2, Color.GRAY);
        renderer.setSeriesShapesVisible(2, false);
        renderer.setSeriesVisibleInLegend(2,Boolean.FALSE);
        xyPlot.setRenderer(renderer);
    }

    @SuppressWarnings("deprecation")
    private void initWindowStile(JFreeChart chart) {
        JFrame frame = new JFrame("График сходимости двух решений");

        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(new WindowListener() {
            public void windowActivated(WindowEvent event) {}

            public void windowClosed(WindowEvent event) {}

            public void windowClosing(WindowEvent event) {
                Object[] options = { "Да", "Нет" };
                int n = JOptionPane
                        .showOptionDialog(event.getWindow(), "Закрыть все окна с графиками?",
                                "Подтверждение", JOptionPane.YES_NO_OPTION,
                                JOptionPane.QUESTION_MESSAGE, null, options,
                                options[0]);
                if (n == 0) {
                    event.getWindow().setVisible(false);
                    System.exit(0);
                }
            }

            public void windowDeactivated(WindowEvent event) {}

            public void windowDeiconified(WindowEvent event) {}

            public void windowIconified(WindowEvent event) {}

            public void windowOpened(WindowEvent event) { }
        });

        frame.getContentPane().add(new ChartPanel(chart));
        frame.setSize(800, 600);
        frame.show();
    }

    private void initWindowContentStyle(JFreeChart chart) {
        Plot plot = chart.getPlot();
        plot.setBackgroundPaint(Color.white);

        XYPlot xyPlot = chart.getXYPlot();
        // Задание отображения координатной сетки
        xyPlot.setDomainGridlinePaint(Color.black);
        xyPlot.setRangeGridlinePaint(Color.black);

        // Перемещаем легенду в верхний правый угол
        LegendTitle lt = new LegendTitle(xyPlot);
        lt.setItemFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
        lt.setBackgroundPaint(Color.white);
        lt.setFrame(new BlockBorder(Color.black));
        lt.setPosition(RectangleEdge.RIGHT);
        XYTitleAnnotation ta = new XYTitleAnnotation(0.98, 0.95, lt, RectangleAnchor.TOP_RIGHT);
        xyPlot.addAnnotation(ta);

        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setSeriesShapesVisible(0, false);
        renderer.setSeriesShapesVisible(1, false);
        renderer.setSeriesPaint(0, Color.GRAY);
        renderer.setSeriesPaint(1, Color.BLACK);
        xyPlot.setRenderer(renderer);
    }

    private void initCoordinateGridStyle(double tMaxBound, double xMaxBound, JFreeChart chart) {
        XYPlot xyPlot = chart.getXYPlot();
        // Задание верхних границ координатных осей
        ValueAxis domainAxis = xyPlot.getDomainAxis();
        domainAxis.setUpperBound(tMaxBound);
        ValueAxis rangeAxis = xyPlot.getRangeAxis();
        rangeAxis.setUpperBound(xMaxBound);

        domainAxis.setTickLabelFont(new Font(Font.SANS_SERIF, Font.PLAIN, 15));
        rangeAxis.setTickLabelFont(new Font(Font.SANS_SERIF, Font.PLAIN, 15));
    }

    private void initCoordinateGridStyle(JFreeChart chart) {
        XYPlot xyPlot = chart.getXYPlot();
        ValueAxis domainAxis = xyPlot.getDomainAxis();
        ValueAxis rangeAxis = xyPlot.getRangeAxis();

        domainAxis.setTickLabelFont(new Font(Font.SANS_SERIF, Font.PLAIN, 15));
        rangeAxis.setTickLabelFont(new Font(Font.SANS_SERIF, Font.PLAIN, 15));
    }

    private XYSeriesCollection initGraphicsCollection(String graphName1, String graphName2,
                                                      double[] t, double[] x1, double[] x2) {
        XYSeries series1 = new XYSeries(graphName1);
        XYSeries series2 = new XYSeries(graphName2);

        int n = t.length;
        for (int i = 0; i < n; i++) {
            series1.add(t[i], x1[i]);
            series2.add(t[i], x2[i]);
        }

        XYSeriesCollection dataSet = new XYSeriesCollection();
        dataSet.addSeries(series1);
        dataSet.addSeries(series2);
        return dataSet;
    }

    private XYSeriesCollection initGraphicsCollection(String graphName1,
                                                      double[] t, double[] x1, double[] x2, double[] x3) {
        XYSeries series1 = new XYSeries(graphName1);
        XYSeries series2 = new XYSeries("major");
        XYSeries series3 = new XYSeries("minor");

        int n = t.length;
        for (int i = 0; i < n; i++) {
            series1.add(t[i], x1[i]);
            series2.add(t[i], x2[i]);
            series3.add(t[i], x3[i]);
        }

        XYSeriesCollection dataSet = new XYSeriesCollection();
        dataSet.addSeries(series1);
        dataSet.addSeries(series2);

        dataSet.addSeries(series3);
        return dataSet;
    }
}
